@@ -1,6 +1,6 @@
 /* MI Command Set.
 
-   Copyright (C) 2000-2013 Free Software Foundation, Inc.
+   Copyright (C) 2000-2014 Free Software Foundation, Inc.
 
    Contributed by Cygnus Solutions (a Red Hat company).
 
@@ -23,6 +23,7 @@
 #include "arch-utils.h"
 #include "target.h"
 #include "inferior.h"
+#include "infrun.h"
 #include <string.h>
 #include "exceptions.h"
 #include "top.h"
@@ -52,9 +53,7 @@
 #include "ctf.h"
 #include "ada-lang.h"
 #include "linespec.h"
-#ifdef HAVE_PYTHON
-#include "python/python-internal.h"
-#endif
+#include "extension.h"
 
 #include <ctype.h>
 #include <sys/time.h>
@@ -1768,7 +1767,7 @@ mi_cmd_data_write_memory_bytes (char *command, char **argv, int argc)
   else
     {
       /* Pattern is longer than or equal to count:
-         just copy len bytes.  */
+         just copy count bytes.  */
       data = databuf;
     }
 
@@ -1820,10 +1819,8 @@ mi_cmd_list_features (char *command, char **argv, int argc)
       ui_out_field_string (uiout, NULL, "undefined-command-error-code");
       ui_out_field_string (uiout, NULL, "exec-run-start-option");
 
-#if HAVE_PYTHON
-      if (gdb_python_initialized)
+      if (ext_lang_initialized_p (get_ext_lang_defn (EXT_LANG_PYTHON)))
 	ui_out_field_string (uiout, NULL, "python");
-#endif
 
       do_cleanups (cleanup);
       return;
@@ -2487,8 +2484,7 @@ mi_cmd_trace_find (char *command, char **argv, int argc)
       return;
     }
 
-  if (current_trace_status ()->running)
-    error (_("May not look at trace frames while trace is running."));
+  check_trace_running (current_trace_status ());
 
   if (strcmp (mode, "frame-number") == 0)
     {
